import datetime
import functools
import os
from _typeshed import Incomplete
from airflow.auth.managers.base_auth_manager import BaseAuthManager as BaseAuthManager
from airflow.exceptions import AirflowConfigException as AirflowConfigException
from airflow.secrets import BaseSecretsBackend as BaseSecretsBackend, DEFAULT_SECRETS_SEARCH_PATH as DEFAULT_SECRETS_SEARCH_PATH
from airflow.utils import yaml as yaml
from airflow.utils.module_loading import import_string as import_string
from airflow.utils.providers_configuration_loader import providers_configuration_loaded as providers_configuration_loaded
from airflow.utils.weight_rule import WeightRule as WeightRule
from builtins import set as _set
from collections.abc import Generator
from configparser import ConfigParser
from typing import Any, IO, Iterable, Pattern
from typing_extensions import overload

log: Incomplete
ConfigType = str | int | float | bool
ConfigOptionsDictType = dict[str, ConfigType]
ConfigSectionSourcesType = dict[str, str | tuple[str, str]]
ConfigSourcesType = dict[str, ConfigSectionSourcesType]
ENV_VAR_PREFIX: str

@overload
def expand_env_var(env_var: None) -> None: ...
@overload
def expand_env_var(env_var: str) -> str: ...
def run_command(command: str) -> str: ...
def retrieve_configuration_description(include_airflow: bool = True, include_providers: bool = True, selected_provider: str | None = None) -> dict[str, dict[str, Any]]: ...

class AirflowConfigParser(ConfigParser):
    configuration_description: Incomplete
    upgraded_values: Incomplete
    is_validated: bool
    def __init__(self, default_config: str | None = None, *args, **kwargs) -> None: ...
    def is_template(self, section: str, key) -> bool: ...
    def get_default_value(self, section: str, key: str, fallback: Any = None, raw: bool = False, **kwargs) -> Any: ...
    def get_default_pre_2_7_value(self, section: str, key: str, **kwargs) -> Any: ...
    @functools.cached_property
    def sensitive_config_values(self) -> _set[tuple[str, str]]: ...
    deprecated_options: dict[tuple[str, str], tuple[str, str, str]]
    many_to_one_deprecated_options: dict[tuple[str, str], list[tuple[str, str, str]]]
    deprecated_sections: dict[str, tuple[str, str]]
    @functools.cached_property
    def inversed_deprecated_options(self): ...
    @functools.cached_property
    def inversed_deprecated_sections(self): ...
    deprecated_values: dict[str, dict[str, tuple[Pattern, str, str]]]
    enums_options: Incomplete
    def get_sections_including_defaults(self) -> list[str]: ...
    def get_options_including_defaults(self, section: str) -> list[str]: ...
    def optionxform(self, optionstr: str) -> str: ...
    def make_sure_configuration_loaded(self, with_providers: bool) -> Generator[None, None, None]: ...
    def write(self, file: IO[str], section: str | None = None, include_examples: bool = True, include_descriptions: bool = True, include_sources: bool = True, include_env_vars: bool = True, include_providers: bool = True, comment_out_everything: bool = False, hide_sensitive_values: bool = False, extra_spacing: bool = True, only_defaults: bool = False, **kwargs: Any) -> None: ...
    def restore_core_default_configuration(self) -> None: ...
    def validate(self) -> None: ...
    def mask_secrets(self) -> None: ...
    def get_mandatory_value(self, section: str, key: str, **kwargs) -> str: ...
    def get_mandatory_list_value(self, section: str, key: str, **kwargs) -> list[str]: ...
    @overload
    def get(self, section: str, key: str, fallback: str = ..., **kwargs) -> str: ...
    @overload
    def get(self, section: str, key: str, **kwargs) -> str | None: ...
    def getboolean(self, section: str, key: str, **kwargs) -> bool: ...
    def getint(self, section: str, key: str, **kwargs) -> int: ...
    def getfloat(self, section: str, key: str, **kwargs) -> float: ...
    def getlist(self, section: str, key: str, delimiter: str = ',', **kwargs): ...
    def getimport(self, section: str, key: str, **kwargs) -> Any: ...
    def getjson(self, section: str, key: str, fallback: Incomplete | None = None, **kwargs) -> dict | list | str | int | float | None: ...
    def gettimedelta(self, section: str, key: str, fallback: Any = None, **kwargs) -> datetime.timedelta | None: ...
    def read(self, filenames: str | bytes | os.PathLike | Iterable[str | bytes | os.PathLike], encoding: Incomplete | None = None): ...
    def read_dict(self, dictionary: dict[str, dict[str, Any]], source: str = '<dict>'): ...
    def has_option(self, section: str, option: str) -> bool: ...
    def set(self, section: str, option: str, value: str | None = None) -> None: ...
    def remove_option(self, section: str, option: str, remove_default: bool = True): ...
    def getsection(self, section: str) -> ConfigOptionsDictType | None: ...
    def as_dict(self, display_source: bool = False, display_sensitive: bool = False, raw: bool = False, include_env: bool = True, include_cmds: bool = True, include_secret: bool = True) -> ConfigSourcesType: ...
    def suppress_future_warnings(self) -> Generator[Incomplete]: ...
    def load_test_config(self) -> None: ...
    def expand_all_configuration_values(self) -> None: ...
    def remove_all_read_configurations(self) -> None: ...
    @property
    def providers_configuration_loaded(self) -> bool: ...
    def load_providers_configuration(self) -> None: ...

def get_airflow_home() -> str: ...
def get_airflow_config(airflow_home: str) -> str: ...
def get_all_expansion_variables() -> dict[str, Any]: ...
def create_default_config_parser(configuration_description: dict[str, dict[str, Any]]) -> ConfigParser: ...
def create_pre_2_7_defaults() -> ConfigParser: ...
def write_default_airflow_configuration_if_needed() -> AirflowConfigParser: ...
def load_standard_airflow_configuration(airflow_config_parser: AirflowConfigParser): ...
def initialize_config() -> AirflowConfigParser: ...
def write_webserver_configuration_if_needed(airflow_config_parser: AirflowConfigParser): ...
def make_group_other_inaccessible(file_path: str): ...
def get(*args, **kwargs) -> ConfigType | None: ...
def getboolean(*args, **kwargs) -> bool: ...
def getfloat(*args, **kwargs) -> float: ...
def getint(*args, **kwargs) -> int: ...
def getsection(*args, **kwargs) -> ConfigOptionsDictType | None: ...
def has_option(*args, **kwargs) -> bool: ...
def remove_option(*args, **kwargs) -> bool: ...
def as_dict(*args, **kwargs) -> ConfigSourcesType: ...
def set(*args, **kwargs) -> None: ...
def ensure_secrets_loaded() -> list[BaseSecretsBackend]: ...
def get_custom_secret_backend() -> BaseSecretsBackend | None: ...
def initialize_secrets_backends() -> list[BaseSecretsBackend]: ...
def __getattr__(name): ...
def initialize_auth_manager() -> BaseAuthManager: ...

AIRFLOW_HOME: Incomplete
AIRFLOW_CONFIG: Incomplete
TEST_DAGS_FOLDER: Incomplete
TEST_PLUGINS_FOLDER: Incomplete
SECRET_KEY: Incomplete
FERNET_KEY: str
WEBSERVER_CONFIG: str
conf: AirflowConfigParser
secrets_backend_list: Incomplete
